/*

Copyright (C) 2006 NTT DATA Corporation

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

 */

package com.clustercontrol.jobmanagement.view.action;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PlatformUI;

import com.clustercontrol.jobmanagement.bean.JobConstant;
import com.clustercontrol.jobmanagement.composite.JobListComposite;
import com.clustercontrol.jobmanagement.composite.JobTreeComposite;
import com.clustercontrol.jobmanagement.dialog.JobDialog;
import com.clustercontrol.jobmanagement.util.JobEditStateUtil;
import com.clustercontrol.jobmanagement.util.JobUtil;
import com.clustercontrol.jobmanagement.view.JobListView;
import com.clustercontrol.ws.jobmanagement.JobTreeItem;

/**
 * ジョブ[一覧]ビューの「変更」のクライアント側アクションクラス<BR>
 * 
 * @version 2.0.0
 * @since 1.0.0
 */
public class ModifyJobAction implements IViewActionDelegate {
	/** アクションID */
	public static final String ID = "com.clustercontrol.jobmanagement.view.action.ModifyJobAction";
	/** ビュー */
	private IViewPart m_view;

	/**
	 * ビューを保持します。ワークベンチにロードされた際に呼ばれます。
	 * 
	 * @param view ビューのインスタンス
	 * 
	 * @see org.eclipse.ui.IViewActionDelegate#init(org.eclipse.ui.IViewPart)
	 */
	@Override
	public void init(IViewPart view) {
		m_view = view;
	}

	/**
	 * ジョブ[一覧]ビューの「変更」が押された場合に、ジョブを変更します。
	 * <p>
	 * <ol>
	 * <li>ジョブ[一覧]ビューから選択されたジョブツリーアイテムを取得します。</li>
	 * <li>ジョブ[一覧]ビューから選択されたジョブツリーアイテムの親を取得します。</li>
	 * <li>選択したジョブツリーアイテムのジョブ種別に合わせて、ジョブ[ジョブの作成・変更]ダイアログを表示します。</li>
	 * <li>ジョブ[一覧]ビューを更新します。</li>
	 * </ol>
	 * 
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 * @see com.clustercontrol.jobmanagement.dialog.JobDialog
	 * @see com.clustercontrol.jobmanagement.view.JobListView
	 * @see com.clustercontrol.jobmanagement.composite.JobTreeComposite
	 * @see com.clustercontrol.jobmanagement.composite.JobListComposite
	 */
	@Override
	public void run(IAction action) {
		JobTreeItem item = null;
		JobTreeItem parent = null;
		int focus = 0;

		if (m_view instanceof JobListView) {
			JobListView view = (JobListView) m_view
					.getAdapter(JobListView.class);
			JobTreeComposite tree = view.getJobTreeComposite();
			JobListComposite list = view.getJobListComposite();
			if (tree.getTree().isFocusControl()) {
				item = tree.getSelectItem();
				parent = item.getParent();
				focus = 0;
			} else if (list.getTable().isFocusControl()) {
				item = list.getSelectJobTreeItem();
				if (item instanceof JobTreeItem) {
					parent = item.getParent();
				}
				focus = 1;
			} else {
				return;
			}

			if (parent instanceof JobTreeItem && item instanceof JobTreeItem) {
				boolean readOnly = !JobEditStateUtil.isLockedJobunitId(item.getData().getJobunitId());
				JobDialog dialog = new JobDialog(
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
						readOnly);
				dialog.setJobTreeItem(item);

				//ダイアログ表示
				if (dialog.open() == IDialogConstants.OK_ID) {
					if (!readOnly) {
						// 編集できるときにOKが押されたときは、設定が変更されたと判定する
						JobEditStateUtil.addEditedJobunit(item);
						if (item.getData().getType() == JobConstant.TYPE_JOBUNIT) {
							JobUtil.setJobunitIdAll(item, item.getData().getJobunitId());
						}
					}
				}

				if (focus == 0) {
					tree.getTreeViewer().sort(parent);
					tree.refresh(parent);
					tree.refresh(parent);
					tree.getTreeViewer().setSelection(
							new StructuredSelection(item), true);
				} else {
					tree.getTreeViewer().sort(parent);
					tree.refresh(parent);
					tree.getTreeViewer().setSelection(
							new StructuredSelection(parent), true);
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IAction action, ISelection selection) {

	}
}