//
// This file is part of the joeSNMP Java Library.
//
// joeSNMP is Copyright (C) 2002-2003 Blast Internet Services, Inc.  All rights reserved.
// joeSNMP is a derivative work, containing both original code, included code and modified
// code that was published under the GNU Lesser General Public License. Copyrights for modified 
// and included code are below.
//
// Original code base Copyright (C) 1999-2001 Oculan Corp.  All rights reserved.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// See: http://www.fsf.org/copyleft/lesser.html
//
// For more information contact:
//      joeSNMP Licensing       <joesnmp-license@lists.sourceforge.net>
//      http://sourceforge.net/projects/joesnmp/
//
// Modifications:
//
// $Id: SnmpPortal.java,v 1.6 2005/03/22 19:02:27 aozarov Exp $
//
// 2005 Mar 21: aozarov@hotmail.com
//      Enable to set receive/send UDP buffers
//      Fixed fastReceiverQ (was calling wait without obtaining the monitor)
// 
// 2004 Oct 04: krishnaraj@ieee.org
//		Changed the default scope of the thread pool variables & Executor class, 
//		and added the logic to terminate the threads of Thread pool in close()
//
// 2004 Sep 24: krishnaraj@ieee.org
//		Changed the default thread pool size to 0. This means that thread pool
//      is inactive by default, in which case the receiver thread will take care
//      of handle packet. Also a wrapper function invokeHandlePkt has been
//      introduced to handle exceptions in a central location to reduce code.
//
// 2004 Sep 22: krishnaraj@ieee.org
//		Added the feature of thread pooling. There will be a pool of threads and 
//		the free thread will be assigned the received packet for handling. Since
//		No multithread issues are seen in the sequence of handlePkt() function,
//		it is recommended that in the Handlers while processing get/set should
//		implement the syncronization if necessary.
//
// 2004 Sep 19: Dimitris.Andreadis@jboss.org
//		Added the ability to bind to a specific local interface, fixed imports
//
// 2003 Aug 21:
//		Added the ability to generate a new DatagramSocket on a random
//		high number port if needed by setting the port to -1.

package org.opennms.protocols.snmp;

import java.io.ByteArrayOutputStream;
import java.io.InterruptedIOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.lang.reflect.Method;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.util.Date;
import java.util.LinkedList;
import java.util.Vector;

import org.opennms.protocols.snmp.asn1.ASN1;
import org.opennms.protocols.snmp.asn1.AsnDecodingException;
import org.opennms.protocols.snmp.asn1.AsnEncoder;

/**
 * Abstracts the communication related details from the SnmpSession and
 * SnmpTrapSession.
 *
 * @author <a href="mailto:weave@oculan.com">Brian Weaver</a>
 * @author <a href="mailto:sowmya@opennms.org">Sowmya Nataraj</a>
 * @author <a href="http://www.opennms.org">OpenNMS</a>
 * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @author <a href="mailto:krishnaraj@ieee.org"></a>
 *
 * @see SnmpSession
 * @see SnmpTrapSession
 * @see java.net.DatagramSocket
 */
public class SnmpPortal
	extends Object
{
    public static final int DEFAULT_THREADPOOL_SIZE=0;
    public static final int MAXIMUM_THREADPOOL_SIZE = 12;
    public static final int DEFAULT_RECEIVE_BUFFER_SIZE = 64 * 1024;
    public static final int DEFAULT_SEND_BUFFER_SIZE = 64 * 1024;
    public static final int DATAGRAM_SOCKET_BUFFER_SIZE = 64 * 1024;
    
	/**
	 * The packet handler that is used to process
	 * received SNMP packets and invalid datagrams.
	 * The handler must also process any exceptions
	 * that occurs in the receiving thread.
	 *
	 */
	private SnmpPacketHandler m_handler;

	/**
	 * The datagram socket used to send and receive
	 * SNMP messages.
	 *
	 */
	private DatagramSocket m_comm;

	/**
	 * the receiver thread that runs the inner class Receiver.
	 */
	private Receiver m_recvThread;

	/**
	 * the number of executor threads in the thread pool
	 * specified during construction of SnmpPortal 
	 */
	private int m_numOfExecutorThreads = DEFAULT_THREADPOOL_SIZE;

	/**
	 * The thread pool that holds the executor threads 
	 */
	private Vector m_threadPool;
	
    /**
     * Cache used bufferes
     */
    private LinkedList m_usedBuffers  = new LinkedList();
        
	/**
	 * ASN.1 encoder used to decode the SNMP messages.
	 * If the decoded fails to decode the specific messages
	 * the is should throw and appropiate ASN.1 exception
	 *
	 */
	private AsnEncoder m_encoder;

	/**
	 * When set the portal object's close method has been
	 * invoked. This is needed since the internal receiver
	 * thread will block on the communication channel. To
	 * "wake" the thread the close() method on the comm channel
	 * is performed. This will cause an exception to be genereated
	 * in the receiver thread. If the value of m_isClosing is true
	 * then the exception is ignored.
	 */
	private volatile boolean m_isClosing;

	/**
	 * workaround for Bug (JOE-22)
	 * 
	 * prevent this race condition.
	 * 
	 * java.net.DatagramSocket#close() : close(fd:10)
	 *  -> java.net.Socket#connect()  : connect(fd:10)
	 *  -> java.net.DatagramSocket#receive() : recv(fd:10)
	 * 
	 * http://issues.opennms.org:8280/browse/JOE-22?page=com.atlassian.jira.plugin.system.issuetabpanels%3Aall-tabpanel#issue-tabs
	 * joeSNMP receiver thread receiving data from non-SNMP sockets
	 */
	private volatile Object closeLock = new Object();
	
	/**
	 * Set to true if it is necessary to set the socket timeout value via the Socket.setSoTimeout()
	 * method in order to keep from blocking indefinitely on a socket I/O call.  This value
	 * is configurable at runtime via the system property
	 * "org.opennms.joeSNMP.vmhacks.socketSoTimeoutRequired".
	 * If this property is set to 'no', the bSocketSoTimeoutRequired variable will be set to
	 * false and the SNMP trap socket timeout will not be set.
	 * If this property is set to 'yes' or the property does not exist, the bSocketSoTimeoutRequired
	 * variable will be set to true. and the socket timeout will be set.
	 * Default value is true.
	 */
	private boolean bSocketSoTimeoutRequired=true;

	/**
	 * Identifies the system property that may be used to specify whether or not a timeout value is set
	 * on the SNMP trap socket.  Valid values are 'yes' and 'no'.
	 */
	private static final String PROP_SOCKET_TIMEOUT_REQUIRED	= "org.opennms.joeSNMP.vmhacks.socketSoTimeoutRequired";
	/**
	 * Identifies the system property that may be used to specify the number of milliseconds to use for the
	 * socket timeout.
	 */
	private static final String PROP_SOCKET_TIMEOUT_PERIOD = "org.opennms.joeSNMP.vmhacks.socketSoTimeoutPeriod";

    private static final Class THREAD_CATEGORY_CLASS = findThreadCategoryClass();
    
    private static Class findThreadCategoryClass()
    {
        Class tmp = null;
        
        try
        {
            tmp = Class.forName("org.opennms.core.utils.ThreadCategory");
        }
        catch (ClassNotFoundException e)
        {
            // Do nothing
        }
        
        return tmp;
    }
     
	/**
	 * Private constructor used to disallow the
	 * default constructor.
	 *
	 * @exception java.lang.UnsupportedOperationException Always thrown!
	 */
	private SnmpPortal()
		throws java.lang.UnsupportedOperationException
	{
		throw new java.lang.UnsupportedOperationException("Illegal constructor call");
	}

	/**
	 * The SnmpPortal constructor. The constructor is used to build a portal
	 * on the specified port, and forward messages to the defined handler.
	 * All messages are decoded using the encoder specified during construction.
	 *
	 * @param handler	The SNMP packet handler.
	 * @param encoder	The ASN.1 codec object.
	 * @param port		The port to send and receive datagram from.
	 *
	 * @exception java.net.SocketException Thrown if an error occurs setting
	 *	up the communication channel.
	 * @exception java.lang.IllegalArgumentException Thrown if any of the parameters
	 *	are null or invalid.
	 *
	 */
	SnmpPortal(SnmpPacketHandler handler, AsnEncoder encoder, int port)
		throws SocketException
	{
		// Call the other constructor with default number of threads and default buffer size
	    this(handler, encoder, port, null, DEFAULT_THREADPOOL_SIZE, DEFAULT_RECEIVE_BUFFER_SIZE, DEFAULT_SEND_BUFFER_SIZE);
	}
	
	/**
	 * The SnmpPortal constructor. The constructor is used to build a portal
	 * on the specified port, and forward messages to the defined handler.
	 * All messages are decoded using the encoder specified during construction.
	 *
	 * @param handler	The SNMP packet handler.
	 * @param encoder	The ASN.1 codec object.
	 * @param port		The port to send and receive datagram from.
     * @param laddr		The local address to bind. 
	 *
	 * @exception java.net.SocketException Thrown if an error occurs setting
	 *	up the communication channel.
	 * @exception java.lang.IllegalArgumentException Thrown if any of the parameters
	 *	are null or invalid.
	 *
	 */
	SnmpPortal(SnmpPacketHandler handler, AsnEncoder encoder, int port, InetAddress laddr)
		throws SocketException
	{
        // Call the other constructor with default number of threads and default buffer size
	    this(handler, encoder, port, laddr, DEFAULT_THREADPOOL_SIZE, DEFAULT_RECEIVE_BUFFER_SIZE, DEFAULT_SEND_BUFFER_SIZE); 
	}

	SnmpPortal(SnmpPacketHandler handler, AsnEncoder encoder, int port, InetAddress laddr, int numOfThreads)
	   throws SocketException 
    {
        // Call the other constructor with default buffers size
        this(handler, encoder, port, laddr, numOfThreads, DEFAULT_RECEIVE_BUFFER_SIZE, DEFAULT_SEND_BUFFER_SIZE);         
    }
    
    /**
     * The SnmpPortal constructor. The constructor is used to build a portal
     * on the specified port, and forward messages to the defined handler.
     * All messages are decoded using the encoder specified during construction.
     *
     * @param handler      The SNMP packet handler.
     * @param encoder      The ASN.1 codec object.
     * @param port         The port to send and receive datagram from.
     * @param laddr        The local address to bind. 
     * @param numOfThreads The number of Executor threads in pool.
     * @param receiveBufferSize Sets the SO_RCVBUF option to the specified value for this DatagramSocket
     * @param sendBufferSize Sets the SO_SNDBUF option to the specified value for this DatagramSocket.
     *
     * @exception java.net.SocketException Thrown if an error occurs setting
     *  up the communication channel.
     * @exception java.lang.IllegalArgumentException Thrown if any of the parameters
     *  are null or invalid.
     *
     */
    SnmpPortal(SnmpPacketHandler handler, AsnEncoder encoder, int port, InetAddress laddr, int numOfThreads,
            int receiveBufferSize, int sendBufferSize) throws SocketException
	{
		if(handler == null || encoder == null)
			throw new IllegalArgumentException("Invalid argument");

		m_handler = handler;

        if (port >= 0) {
		    m_comm = new DatagramSocket(port, laddr);
        }
        else {
            m_comm = new DatagramSocket(0, laddr);
        }

        m_comm.setReceiveBufferSize(receiveBufferSize);
        m_comm.setSendBufferSize(sendBufferSize);
        
        //
		// Determine whether or not it is necessary to use the socket.setSoTimeout()
		// method to set the socket timeout value thereby mimic'ing non-blocking socket I/O.
		// On platforms whose system close() is not preemptive it is necessary to use the socket timeout
		// to keep from blocking indefinitely on any socket call that performs I/O.
		//
		bSocketSoTimeoutRequired = true; // Default is to use set the socket timeout
		String strSocketSoTimeoutRequired = System.getProperty(PROP_SOCKET_TIMEOUT_REQUIRED);
		String osName = System.getProperty("os.name");

		if (strSocketSoTimeoutRequired != null && strSocketSoTimeoutRequired.equals("no"))
		{
			bSocketSoTimeoutRequired = false;
		}

		if (bSocketSoTimeoutRequired == true)
		{
			String strSocketSoTimeoutPeriod = System.getProperty(PROP_SOCKET_TIMEOUT_PERIOD);
			int timeout = 100; // Default socket timeout is 100 milli seconds
			if (strSocketSoTimeoutPeriod != null)
			{
				try
				{
					timeout = Integer.parseInt(strSocketSoTimeoutPeriod);
				}
				catch (NumberFormatException e)
				{
					timeout = 100;
				}
			}
			m_comm.setSoTimeout(timeout);
		}
		else if(osName != null && osName.equalsIgnoreCase("linux"))
		{
			// we must force this issue because we do not know
			// what VM there running in. If there running in
			// Sun JDK 1.3.1 with J2SE_PREEMPTCLOSE set then
			// this is unnecessary. If there not running in 1.3.1
			// and the're on linux then THEY MUST have a timeout
			// set of it will hang a thread.
			//
			m_comm.setSoTimeout(100);
		}

		m_isClosing  = false;
		
		// Check for the boundary conditions for requested number of threads
		if( 0 < numOfThreads && numOfThreads <= MAXIMUM_THREADPOOL_SIZE )
		{
         m_numOfExecutorThreads = numOfThreads;
		}

		
		m_threadPool = new Vector(m_numOfExecutorThreads);
		//Thread Pool will be created only if the m_numOfExecutorThreads is greater than ZERO
		//Otherwise the Receiver thread itself will handle the packet.
		for(int nThreadID = 0; nThreadID < m_numOfExecutorThreads; nThreadID++)
		{
			Executor fastExecutor = new Executor("SnmpFastExecutor-" + (nThreadID+1));
			m_threadPool.add(fastExecutor);	
			fastExecutor.start();			
		}

		m_recvThread = new Receiver("SnmpPortal-" + m_comm.getPort());
		m_encoder    = encoder;

		m_recvThread.start();

	}
	
	
	/**
	 * Defines the inner class, Executor threas, which 
	 * will be part of thread pool that is used for 
	 * handling the packet 
	 * 
	 */	
	private class Executor extends Thread
	{		
		DatagramPacket pkt;
		
		public Executor(String str) 
		{
			super(str);
		}
		
		public void run()
		{
			while(!m_isClosing)
			{
				try
				{
					if(null != pkt)
                    {
                        try
                        {
                            invokeHandlePkt(pkt);
                        }
                        catch (Throwable e) 
                        {
                            e.printStackTrace();
                        }
                        finally
                        {
                            pkt = null;
                        }
                    }
                    
                    synchronized(this)
					{
						while(!m_isClosing && pkt == null)
							wait();
					}
				}
				catch (InterruptedException e) 
				{
                    
                    // ignore (probably need to close)
				}
			}
		}
		
        synchronized boolean acceptPacket(DatagramPacket newPkt)
        {
            if (pkt != null)
                return false;
            
            pkt = newPkt;
            notify();
            return true;
        }

        synchronized void close()
        {
            // assuming m_isClosing is set to true
            notify();
        }
	}
	
	
	/**
	 * Defines the inner class that monitors the
	 * datagram socket and receives all the PDU responses.
	 * If an exception is generated then it is saved
	 * in m_why and can be re-generated with a call
	 * to raise().
	 *
	 */
	private class Receiver extends Thread
	{
        private final LinkedList m_fastReceiverQ = new LinkedList();
        private final Thread m_fastReceiver = new Thread(new Runnable()
        {
            public void run()
            {
                LinkedList fastReceiverQ = m_fastReceiverQ;
                while(!m_isClosing && Thread.interrupted() == false)
                {
                    byte[] buf = getBuffer();

                    try
                    {
                        DatagramPacket pkt = new DatagramPacket(buf, buf.length);
                        // workaround for Bug (JOE-22) : synchronized + isClosed 
                        synchronized (closeLock) {
                        	if (m_comm.isClosed()) {
                        		break;
                        	}
                        	m_comm.receive(pkt);
                        }
                        synchronized(fastReceiverQ)
                        {
                            fastReceiverQ.addLast(pkt);
                            
                            if (fastReceiverQ.size() == 1)
                                fastReceiverQ.notify();
                        }
                    }
                    catch(InterruptedIOException ioe)
                    {
                        // recycle the buffer if possible
                        cacheBuffer(buf);
                    }
                    catch(Exception e)
                    {
                        if(!m_isClosing)
                        {
                            // recycle the buffer if possible
                            cacheBuffer(buf);

                            if (THREAD_CATEGORY_CLASS != null)
                            {
                                boolean handled = true;
                                try
                                {
                                    
                                    Class[] methodParmList = { Class.class };
                                    Method loggerM = THREAD_CATEGORY_CLASS.getMethod("getInstance", methodParmList);

                                    Object[] parmList = { this.getClass() };
                                    Object loggerI = loggerM.invoke(null, parmList);

                                    methodParmList = new Class[] { Object.class, Throwable.class };
                                    Method infoM   = THREAD_CATEGORY_CLASS.getMethod("info", methodParmList);

                                    parmList = new Object[] { "An unknown error occured decoding the packet", e };
                                    infoM.invoke(loggerI, parmList);
                                }
                                catch(Throwable t)
                                {
                                    handled = false;
                                }

                                if(!handled)
                                {
                                    System.out.println(new Date() + " - Exception: " + e.getMessage());
                                }
                            }
                            
                            m_handler.processException(e);
                        }
                    }
                }
            }
        }, Thread.currentThread().getName() + "-FastReceiver");
        
        public Receiver(String name)
        {
            super(name);
        }

        private void notifyOnClose()
        {
            synchronized(m_fastReceiverQ)
            {
                m_fastReceiverQ.notifyAll();
            }
        }
        
		/**
		 * The run method is an infinite loop method that
		 * receives all datagrams for the session. If an
		 * unrecoverable error occurs then the m_handler is
		 * informed of the error
		 *
		 * If a pdu is recovered from the channel then the associated
		 * handler is invoked to process the pdu.
		 *
		 * @see SnmpPacketHandler
		 */
		public void run()
		{
			m_fastReceiver.start();
			LinkedList fastReceiverQ = m_fastReceiverQ;
			//
			// get a buffer for the datagrams
			//
			while(!m_isClosing)
			{
				DatagramPacket pkt = null;
                
				try
				{
					//
					// reset the packet's length
					//
					synchronized(fastReceiverQ)
					{
						while(fastReceiverQ.isEmpty() && !m_isClosing)
							fastReceiverQ.wait();

						if(!m_isClosing)
    						pkt = (DatagramPacket) fastReceiverQ.removeFirst();
					}

                    
                    if(null != pkt)
                    {
                        //System.out.println(new Date() + "PKT Contents are :: " + pkt.toString());
                        
    					if(m_numOfExecutorThreads > 0)
    					{
                            boolean wasAccepted = false;
                            
                            for (int nThreadIndex = 0; nThreadIndex < m_numOfExecutorThreads; ++nThreadIndex)
                            {
                                Executor fastExecutor = (Executor) m_threadPool.elementAt(nThreadIndex);
                                
                                if (fastExecutor.acceptPacket(pkt))
                                {
                                    wasAccepted = true;
                                    break;
                                }
                            }
                                
                            // Thread pool is full (use the Receiver thread instead to handle the packet) 
                            if (!wasAccepted)
                                invokeHandlePkt(pkt);    				
                        }
    					else
    					{
    						// Thread pool is not used so Receiver thread should handle the packet 
    						invokeHandlePkt(pkt);
    					}	
                    }
				}
				catch(Exception e)
				{
					if(!m_isClosing)
					{
						boolean handled = true;
                        
                        if (THREAD_CATEGORY_CLASS != null)
                        {
    						try
    						{
    							Class[] methodParmList = { Class.class };
    							Method loggerM = THREAD_CATEGORY_CLASS.getMethod("getInstance", methodParmList);
    
    							Object[] parmList = { this.getClass() };
    							Object loggerI = loggerM.invoke(null, parmList);
    
    							methodParmList = new Class[] { Object.class, Throwable.class };
    							Method infoM   = THREAD_CATEGORY_CLASS.getMethod("info", methodParmList);
    
    							parmList = new Object[] { "An unknown error occured decoding the packet", e };
    							infoM.invoke(loggerI, parmList);
    						}
    						catch(Throwable t)
    						{
    							handled = false;
    						}
    
    						if(!handled)
    						{
    							System.out.println(new Date() + " - Exception: " + e.getMessage());
    						}
                        }
                        
						m_handler.processException(e);
					}
				}
			}
		} // end run()
	}

	/**
	 * Wrapper for invoking the handlePkt method and handling the necessary exceptions.
	 *
	 * @param pkt	 The datagram packet to be decoded
	 *
	 * @exception none
	 *
	 * @see handlePkt
	 *
	 */
	void invokeHandlePkt(DatagramPacket pkt)
	{
		try
		{
			handlePkt(pkt);
		}
		catch(SnmpPduEncodingException err)
		{
            if (THREAD_CATEGORY_CLASS != null)
            {
    			boolean handled = true;
    			try
    			{
    
    				Class[] methodParmList = { Class.class };
    				Method loggerM = THREAD_CATEGORY_CLASS.getMethod("getInstance", methodParmList);
    
    				Object[] parmList = { this.getClass() };
    				Object loggerI = loggerM.invoke(null, parmList);
    
    				methodParmList = new Class[] { Object.class, Throwable.class };
    				Method infoM   = THREAD_CATEGORY_CLASS.getMethod("info", methodParmList);
    
    				parmList = new Object[] { "An error occured decoding the protocol data unit", err };
    				infoM.invoke(loggerI, parmList);
    
    				methodParmList = new Class[0];
    				Method debugEnabledM = THREAD_CATEGORY_CLASS.getMethod("isDebugEnabled", methodParmList);
    
    				parmList = new Object[0];
    				Boolean isEnabled = (Boolean) debugEnabledM.invoke(loggerI, parmList);
    
    				if(isEnabled.booleanValue())
    				{
    					methodParmList = new Class[] { Object.class };
    					Method debugM = THREAD_CATEGORY_CLASS.getMethod("debug", methodParmList);
    
    					OutputStream ostream = new ByteArrayOutputStream();
    					SnmpUtil.dumpHex(new PrintStream(ostream), pkt.getData(), 0, pkt.getLength());
    
    					parmList = new Object[] { ostream };
    					debugM.invoke(loggerI, parmList);
    				}
    			}
    			catch(Throwable t)
    			{
    				handled = false;
    			}
    
    			if(!handled)
    			{
    				System.out.println(new Date() + " - SnmpPortal.Receiver.run: SnmpPduEncodingException: " + err.getMessage());
    				SnmpUtil.dumpHex(System.out, pkt.getData(), 0, pkt.getLength());
    			}
            }
            
			m_handler.processBadDatagram(pkt);
		}
		catch(AsnDecodingException err)
		{
            if (THREAD_CATEGORY_CLASS != null)
            {
    			boolean handled = true;
                
    			try
    			{
    				Class[] methodParmList = { Class.class };
    				Method loggerM = THREAD_CATEGORY_CLASS.getMethod("getInstance", methodParmList);
    
    				Object[] parmList = { this.getClass() };
    				Object loggerI = loggerM.invoke(null, parmList);
    
    				methodParmList = new Class[] { Object.class, Throwable.class };
    				Method infoM   = THREAD_CATEGORY_CLASS.getMethod("info", methodParmList);
    
    				parmList = new Object[] { "An ASN.1 error occured decoding the packet", err };
    				infoM.invoke(loggerI, parmList);
    
    				methodParmList = new Class[0];
    				Method debugEnabledM = THREAD_CATEGORY_CLASS.getMethod("isDebugEnabled", methodParmList);
    
    				parmList = new Object[0];
    				Boolean isEnabled = (Boolean) debugEnabledM.invoke(loggerI, parmList);
    
    				if(isEnabled.booleanValue())
    				{
    					methodParmList = new Class[] { Object.class };
    					Method debugM = THREAD_CATEGORY_CLASS.getMethod("debug", methodParmList);
    
    					OutputStream ostream = new ByteArrayOutputStream();
    					SnmpUtil.dumpHex(new PrintStream(ostream), pkt.getData(), 0, pkt.getLength());
    
    					parmList = new Object[] { ostream };
    					debugM.invoke(loggerI, parmList);
    				}
    			}
    			catch(Throwable t)
    			{
    				handled = false;
    			}
    
    			if(!handled)
    			{
    				System.out.println(new Date() + " - SnmpPortal.Receiver.run: AsnEncodingException: " + err.getMessage());
    				SnmpUtil.dumpHex(System.out, pkt.getData(), 0, pkt.getLength());
    			}
            }
            
			m_handler.processBadDatagram(pkt);
		}
        
        // recycle the packet buffer if possible
        cacheBuffer(pkt.getData());
	}

    byte[] getBuffer()
    {
        synchronized(m_usedBuffers)
        {
            if(!m_usedBuffers.isEmpty())
                return (byte[]) m_usedBuffers.removeFirst();
        }
        
        return new byte[DATAGRAM_SOCKET_BUFFER_SIZE];
    }
    
    void cacheBuffer(byte[] buffer)
    {
        synchronized(m_usedBuffers)
        {
            // only keep 20 * 64k of buffers around
            if(m_usedBuffers.size() < 20)
                m_usedBuffers.addLast(buffer);
        }                
    }
    
	/**
	 * Recovers a SnmpPduPacket or SnmpPduTrap from the passed datagram
	 * and calls the appropriate method in the handler.
	 *
	 * If an error occurs recovering the packet then an exception is
	 * generated. The pdu can be one of SnmpPduRequest or SnmpPduBulk.
	 * The internal session AsnEncoder defined in the SnmpParameters is
	 * used to recover the pdu.
	 *
	 * @param pkt	The datagram packet to be decoded
	 *
	 * @exception SnmpPduEncodingException	Thrown if a pdu or session level error occurs
	 * @exception AsnDecodingException	Thrown if the AsnEncoder encounters an error
	 *
	 * @see SnmpPduTrap
	 * @see SnmpPduPacket
	 * @see SnmpPduRequest
	 * @see SnmpPduBulk
	 * @see SnmpParameters
	 * @see org.opennms.protocols.snmp.asn1.AsnEncoder
	 *
	 */
	private void handlePkt(DatagramPacket pkt)
		throws	SnmpPduEncodingException, AsnDecodingException
	{
		//
		// first decode the header
		//
		byte[] buf = pkt.getData();
		int offset = 0;


		//
		// Decode the ASN.1 header from the front
		// of the SNMP message.
		//
		Object[] rVals = m_encoder.parseHeader(buf, offset);

		//
		// get the return vals
		//
		offset		= ((Integer)rVals[0]).intValue();
		byte asnType	= ((Byte)rVals[1]).byteValue();
		int asnLength	= ((Integer)rVals[2]).intValue();

		//
		// check the ASN.1 Type
		//
		if(asnType != (ASN1.SEQUENCE | ASN1.CONSTRUCTOR))
			throw new AsnDecodingException("Invalid SNMP ASN.1 type");

		//
		// Check the length of the datagram packet
		//
		if(asnLength > pkt.getLength() - offset)
		{
			throw new SnmpPduEncodingException("Insufficent data in packet");
		}

		//
		// get the snmp version.
		//
		SnmpInt32 int32 = new SnmpInt32();
		offset = int32.decodeASN(buf, offset, m_encoder);

		//
		// check the version
		//
		if(int32.getValue() != SnmpSMI.SNMPV1 && int32.getValue() != SnmpSMI.SNMPV2)
		{
			throw new SnmpPduEncodingException("Invalid protocol version");
		}

		//
		// need to get the community
		// Postpone the community check until the pdu
		// has been recovered to determine which community
		// string needs to be verified against.
		//
		SnmpOctetString community = new SnmpOctetString();
		offset = community.decodeASN(buf, offset, m_encoder);

		//
		// get the pdu header, but DO NOT modify the offset
		// in effect we are peeking into the remainder of the
		// packet
		//
		rVals = m_encoder.parseHeader(buf, offset);

		//
		// The command should be sign extended to a
		// negative number. Thus add 256 to wrap it
		//
		int cmd = ((Byte)rVals[1]).intValue() + 256;

		//
		// Now process the Protocol Data Unit
		//
		switch(cmd)
		{
		case SnmpPduPacket.SET:
		case SnmpPduPacket.GET:
		case SnmpPduPacket.GETNEXT:
		case SnmpPduPacket.RESPONSE:
		case SnmpPduPacket.INFORM:
		case SnmpPduPacket.V2TRAP:
		case SnmpPduPacket.REPORT:
			{
				SnmpPduPacket pdu = new SnmpPduRequest();
				offset = pdu.decodeASN(buf, offset, m_encoder);
				m_handler.processSnmpMessage(pkt.getAddress(),		// From Who?
							     pkt.getPort(),		// What Port?
							     int32,			// What Version
							     community,			// Community
							     cmd,			// Snmp Command (Wrapped!)
							     pdu);			// The Protocol Data Unit
			}
			break;

		case SnmpPduPacket.GETBULK:
			{
				SnmpPduPacket pdu = new SnmpPduBulk();
				offset = pdu.decodeASN(buf, offset, m_encoder);
				m_handler.processSnmpMessage(pkt.getAddress(),		// From Who?
							     pkt.getPort(),		// Port
							     int32,			// Version
							     community,			// Community
							     cmd,			// Command (positive wrapped)
							     pdu);			// Protocol Data Unit
			}
			break;

		case SnmpPduTrap.TRAP:
			{
				SnmpPduTrap trap=new SnmpPduTrap();
				offset = trap.decodeASN(buf, offset, m_encoder);
				m_handler.processSnmpTrap(pkt.getAddress(),
							  pkt.getPort(),
							  community,
							  trap);
			}
			break;

		default:
			throw new SnmpPduEncodingException("No matching PDU type found");
		}
	}

	/**
	 * Transmits the passed buffer to the respective peer
	 * agent. If a failure occurs then an IOException is
	 * thrown.
	 *
	 * @param peer		The SNMP peer destination
	 * @param buf		The buffer to transmit.
	 * @param length	The valid length of the buffer
	 *
	 * @exception java.lang.IOException For more details see
	 *	java.net.DatagramSocket.
	 *
	 * @see java.net.DatagramSocket
	 *
	 */
	void send(SnmpPeer peer, byte[] buf, int length)
		throws java.io.IOException
	{
		//
		// create a new datagram packet
		//
		DatagramPacket pkt = new DatagramPacket(buf,
							length,
							peer.getPeer(),
							peer.getPort());

		m_comm.send(pkt);

	}

	/**
	 * Transmits the passed buffer to the respective peer
	 * agent. If a failure occurs then an IOException is
	 * thrown.
	 *
	 * @param peer		The SNMP peer destination
	 * @param buf		The buffer to transmit.
	 *
	 * @exception java.lang.IOException For more details see
	 *	java.net.DatagramSocket.
	 *
	 * @see java.net.DatagramSocket
	 *
	 */
	void send(SnmpPeer peer, byte[] buf)
		throws java.io.IOException
	{
		send(peer, buf, buf.length);
	}

	/**
	 * Sets the default SnmpPacketHandler.
	 *
	 * @param hdl	The new handler
	 *
	 */
	void setPacketHandler(SnmpPacketHandler hdl)
	{
		if(hdl == null)
			throw new IllegalArgumentException("The packet handler must not be null");

		m_handler = hdl;
	}

	/**
	 * Gets the default SnmpPacketHandler for the session.
	 *
	 * @return the SnmpPacketHandler
	 */
	SnmpPacketHandler getPacketHandler()
	{
		return m_handler;
	}

	/**
	 * Sets the default encoder.
	 *
	 * @param encoder	The new encoder
	 *
	 */
	void setAsnEncoder(AsnEncoder encoder)
	{
		if(encoder == null)
			throw new IllegalArgumentException("The ASN.1 codec must not be null");

		m_encoder = encoder;
	}

	/**
	 * Gets the AsnEncoder for the session.
	 *
	 * @return the AsnEncoder
	 */
	AsnEncoder getAsnEncoder()
	{
		return m_encoder;
	}

	/**
	 * Returns true if this portal has had it's <CODE>close</CODE>
	 * method called.
	 *
	 */
	boolean isClosed()
	{
		return m_isClosing;
	}

	/**
	 * Used to close the session. Once called the session should
	 * be considered invalid and unusable.
	 *
	 */
	void close()
	{
		m_isClosing = true;
		// workaround for Bug (JOE-22) : synchronized
		synchronized (closeLock) {
			m_comm.close();
		}
        m_recvThread.notifyOnClose();
        
		try
		{
			//Now try to close the threads in pool just by invoking resume...
			//The threads will die as the m_closing flag is set
			if(0 < m_numOfExecutorThreads)
			{
				for(int nThreadIndex = 0; nThreadIndex < m_numOfExecutorThreads; ++nThreadIndex)
				{
					Executor fastExecutor = (Executor)m_threadPool.elementAt(nThreadIndex);
                    fastExecutor.close();
                    fastExecutor.join();
				}
			}

			// make sure that the caller thread
			// is not the one we are trying to
			// join!
			//
			if(m_recvThread.equals(Thread.currentThread()) == false)
			{
				m_recvThread.join();
			}
		}
		catch(InterruptedException err)
		{
			Thread.currentThread().interrupt(); // reset the flag
		}
	}
}

